/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for microcontrollers PIC (Microchip) and 8-bit AVR (Atmel)
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *                  http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:           osa_piccpro16e.c
 *
 *  Compilers:      HT-PICC PRO 9.70 and higher
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    PICC16F1xxx specific functions overloads
 *
 *  History:        10.02.2010
 *                  10.04.2010 -    3. Bug fixed in _OS_ClrReadySetClrCanContinue: OS_ENABLE_TTIMER->OS_ENABLE_TTIMERS
 *
 ***********************************************************************************************
 */





#define OS_Init_ProcSpec()                      \
    if (_pclath & 0x80)                         \
    {                                           \
        OS_SchedRetPoint();                     \
        OS_ClearReady();                        \
        OS_ClearReadySetCanContinue();          \
        _OS_CheckEvent(0);                      \
        __OS_JumpToTask();                      \
    }

#define _OS_SetPC()         asm("   dw      0x0013      ");     /* moviw  INDF0--   */  \
                            asm("   dw      0x008A      ");     /* movwf  PCLATH    */  \
                            asm("   dw      0x0013      ");     /* moviw  INDF0--   */  \
                            asm("   dw      0x0082      ");     /* movwf  PCL       */  \

#define _OS_SavePC()        asm("   movlb   0x1F        ");                             \
                            asm("   dw      0x086F      ");     /* movf   TOSH, w   */  \
                            asm("   dw      0x001B      ");     /* movwi  INDF0--   */  \
                            asm("   dw      0x086E      ");     /* movf   TOSL, w   */  \
                            asm("   dw      0x001B      ");     /* movwi  INDF0--   */  \
                            asm("   movlb   0x00        ");                             \
 

/************************************************************************************************
 *                                                                                              *
 *                           K E R N E L   F U N C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************/

void _OS_ClrReadySetClrCanContinue (void);

void __OS_JumpToTask (void)
{
    _OS_SET_FSR_CUR_TASK_W(2);
    _OS_SetPC();
}



/*
 ********************************************************************************
 *                                                                              *
 *  void OS_SchedRetPoint (void)                                                *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Return to sheduler with saving current task return point    *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void OS_SchedRetPoint (void)
{
    _OS_SET_FSR_CUR_TASK_W(2);
    _OS_SavePC();
    _OS_bTaskReady = 1;
    _PIC16E_POP();
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReady (void)                                                  *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag and return to the sheduler with seaving   *
 *                  task return point                                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReady (void)
{
    _OS_temp = 0;
    _OS_ClrReadySetClrCanContinue();
    NOP();  // To prevent calling function through GOTO instruction (see previous line)
}

/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClearReadySetCanContinue (void)                                    *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Clear bReady flag, set bCanContinue flag and return to the  *
 *                  sheduler with seaving task return point                     *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 ********************************************************************************
 */

void OS_ClearReadySetCanContinue (void)
{
    _OS_temp = 0xFF;
    _OS_ClrReadySetClrCanContinue();
    NOP();  // To prevent calling function through GOTO instruction (see previous line)
}


/*
 ********************************************************************************
 *                                                                              *
 *  void _OS_ClrReadySetClrCanContinue (void)                                   *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Addition function. Called only from:                        *
 *                      OS_ClearReady()                                         *
 *                      OS_ClearReadySetCanContinue()                           *
 *                  Stack has pointer of task who called theese services.       *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 ********************************************************************************
 */

void _OS_ClrReadySetClrCanContinue (void)
{
    _PIC16E_POP();

    OS_Flags.bEventError = 0;
    
    _OS_SET_FSR_CUR_TASK_W(2);
    _OS_SavePC();

    _OS_bTaskReady = 0;

    #if defined(OS_ENABLE_TTIMERS)  /*100420.3*/

        if (!_OS_bTaskDelay) _OS_bTaskCanContinue = 0;
        if (_OS_temp & 0x01) _OS_bTaskCanContinue = 1;

    #endif
}






/************************************************************************************************
 *                                                                                              *
 *  MAKE FSR POINTER TO OS_CUR_TASK                                                             *
 *                                                                                              *
 *  (for code reducing)                                                                         *
 *                                                                                              *
 ************************************************************************************************/

/**
void _OS_SET_FSR_CUR_TASK (void)
{
    _fsr = (_FSR_TYPE) _OS_CurTask;
}

void _OS_SET_FSR_CUR_TASK_W (char temp)
{
    _fsr = (_FSR_TYPE) _OS_CurTask + temp;
}

/**/













/*
 ************************************************************************************************
 *                                                                                              *
 *                            C O U N T I N G    S E M A P H O R E S                            *
 *                                                                                              *
 ************************************************************************************************
 */







/*
 ************************************************************************************************
 *                                                                                              *
 *                         C R I T I C A L   S E C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------


/*
 ********************************************************************************
 *                                                                              *
 *  void OS_EnterCriticalSection (void)                                         *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Enter critical section.                                     *
 *                  This function disables interrupts (with saving current      *
 *                  state) and sets system flag bInCriticalSection              *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_EnterCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_EnterCriticalSection (void)
{
    __osa_word temp;
    temp = OS_DI();
    OS_Flags.bInCriticalSection = 1;

    OS_Flags.bGIE_CTemp = 0;
    if (temp & 0x80) OS_Flags.bGIE_CTemp = 1;

}



/*
 ********************************************************************************
 *                                                                              *
 *   void OS_LeaveCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Leave critical section.                                     *
 *                  This function restore interrupt state from OS_Flag temp bits*
 *                  and clears system flag bInCriticalSection                   *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define OS_LeaveCriticalSection_DEFINED
//------------------------------------------------------------------------------

void OS_LeaveCriticalSection (void)
{
    char temp;
    OS_Flags.bInCriticalSection = 0;
    temp = 0;
    if (OS_Flags.bGIE_CTemp) temp |= 0x80;
    OS_RI(temp);
}

//------------------------------------------------------------------------------
#endif      //     #ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------
























/************************************************************************************************
 *                                                                                              *
 *                            S Y S T E M   F U N C T I O N S                                   *
 *                                                                                              *
 ************************************************************************************************/







/*
 ********************************************************************************
 *                                                                              *
 *  char OS_DI (void)                                                           *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Disable interrupt with GIE saving                           *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      char - bit 7 = previous GIE value, bits 0..6 = 0            *
 *                                                                              *
 ********************************************************************************
 */

//-------------------------------------------------------------------------------
#define OS_DI_DEFINED
//-------------------------------------------------------------------------------

char OS_DI (void)
{
    char temp;
    temp = _intcon & 0x80;
    _intcon &= 0x7F;
    return temp;
}


//------------------------------------------------------------------------------
#ifdef OS_ENABLE_INT
//------------------------------------------------------------------------------

void OS_DI_INT (void)
{
    char temp;
    _OS_tempI = _intcon & 0x80;
    _intcon &= 0x7F;
    return temp;
}

void OS_RI_INT (void)
{
    if (_OS_tempI & 0x80) _intcon |= 0x80;
}


//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------









/*
 ********************************************************************************
 *                                                                              *
 *  bit _OS_CheckEvent (char bEvent)                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      1 - when condition is true, or timeout occured              *
 *                  0 - condition is false and no timeout                       *
 *                                                                              *
 *  note:           This function overloads "osa.c"                             *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#define _OS_CheckEvent_DEFINED
//------------------------------------------------------------------------------

void _OS_CheckEvent (__osa_word bEvent)
{
    //------------------------------------------------------------------------------
    #ifdef OS_DISABLE_PRIORITY                  // *91103.4*
    //------------------------------------------------------------------------------
        
        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent) 
        {
            #ifdef OS_ENABLE_TTIMERS

                _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state

            #endif
            return;
    
        }
    
        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                OS_Flags.bTimeout = 1;
                return;
            }
    
        #endif
    
        _PIC16E_POP();

    //------------------------------------------------------------------------------
    #else
    //------------------------------------------------------------------------------

        OS_Flags.bTimeout = 0;
        _OS_SET_FSR_CUR_TASK();
    
        if (bEvent) 
        {
            if (_OS_bTaskReady) 
            {
                #ifdef OS_ENABLE_TTIMERS
    
                    _OS_bTaskDelay = 0; // Clear bDelay in temporary variable OS_state
    
                #endif
    
                return;
            }
    
            _OS_bTaskReady = 1;
    
        } else {
            _OS_bTaskReady = 0;
        }
    
        #ifdef OS_ENABLE_TTIMERS
    
            if (_OS_bTaskTimeout && _OS_bTaskCanContinue)
            {
                _OS_bTaskReady = 1;         // *91103.3*
                OS_Flags.bTimeout = 1;
                return;
            }
    
        #endif
    
        _PIC16E_POP();

    //------------------------------------------------------------------------------
    #endif
    //------------------------------------------------------------------------------

}


    

        


