/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for microcontrollers PIC (Microchip) and 8-bit AVR (Atmel)
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *                  http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:           osa_piccpro16e.h
 *
 *  Compilers:      HT-PICC PRO
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    PICC161xxx specific definitions
 *
 *  History:        10.02.2010
 *
 ***********************************************************************************************
 */


#ifndef __OSAPICC16E_H__
#define __OSAPICC16E_H__


#include <htc.h>



/************************************************************************************************
 *                                                                                              *
 *     Integer types definitions                                                                *
 *                                                                                              *
 ************************************************************************************************/

typedef unsigned char   __osa_uint8;
typedef unsigned int    __osa_uint16;
typedef unsigned long   __osa_uint32;
typedef __osa_uint8     __osa_word;
typedef bit             __osa_bool;
#define _OST_INT_MASK    7
#define _OST_INT_SHIFT   3
#define _OST_INT_SIZE    8


/************************************************************************************************
 *                                                                                              *
 *     Location keywords                                                                        *
 *                                                                                              *
 ************************************************************************************************/

#define NEAR            near
#define bank0           near
#define __osa30_near
#define _OS_CUR_FLAGS_IN_FSR        // Work with tasks state througth FSR


/************************************************************************************************
 *                                                                                              *
 *     Processor specified definitions for system flags                                         *
 *     (see OST_SYSTEM_FLAGS and OST_TCB definition in file "OSA.h")                            *
 *                                                                                              *
 ************************************************************************************************/

#define _OST_SYSTEM_FLAGS_PROC_SPEC                                                             \
                                                                                                \
        /*5*/   __osa_word   bTimeout    : 1; /* Event waiting timeout               */      \
        /*6*/   __osa_word   bGIE_CTemp  : 1; /* Saved GIE while in critical section */      \

#define _OST_TCB_PROC_SPEC


/************************************************************************************************
 *                                                                                              *
 *     Registers definitions                                                                    *
 *                                                                                              *
 ************************************************************************************************/

static volatile unsigned int    _fsr    @ 0x04;
static volatile unsigned char   _fsrl   @ 0x04;    
static volatile unsigned char   _fsrh   @ 0x05;    
static volatile char _indf      @ 0x00;
static volatile char _pcl       @ 0x02;
static volatile char _status    @ 0x03;
static volatile char _bsr       @ 0x08;
static volatile char _pclath    @ 0x0A;
static volatile char _intcon    @ 0x0B;
static volatile char _stkptr    @ 0xFED;
static volatile char _tosl      @ 0xFEE;
static volatile char _tosh      @ 0xFEF;

static volatile       bit       _carry      @((unsigned)&_status*8)+0;
static volatile       bit       _zero       @((unsigned)&_status*8)+2;
static                bit       _irp        @((unsigned)&_status*8)+7;
static                bit       _gie        @((unsigned)&_intcon*8)+7;


#define _PIC16E_POP()   _stkptr--;


/************************************************************************************************
 *                                                                                              *
 *     Constants and types                                                                      *
 *                                                                                              *
 ************************************************************************************************/

#undef  OS_SMSG_SIZE
#define OS_SMSG_SIZE            sizeof(OS_SMSG_TYPE)

typedef unsigned int            _FSR_TYPE;

// Type of code pointer (used in TCB)
typedef unsigned int             OST_CODE_POINTER;






/************************************************************************************************
 *                                                                                              *
 *     Platform macros                                                                          *
 *                                                                                              *
 ************************************************************************************************/

#define    _PIC18_ERRATA_NOP()                      // Empty macro for this type of PICs
#define     OS_CLRWDT()         asm(" clrwdt ");
#define     OS_ClrWdt()         asm(" clrwdt ");
#define     OS_SLEEP()          asm(" sleep  ");
#define     OS_Sleep()          asm(" sleep  ");






/************************************************************************************************
 *                                                                                              *
 *     Context switching macros                                                                 *
 *                                                                                              *
 ************************************************************************************************/


void __OS_JumpToTask (void);


#define _OS_JumpToTask()        __OS_JumpToTask()

#define _OS_SaveRetAddr()       asm("   movlb   0   ")








void OS_SchedRetPoint (void);
void OS_ClearReady (void);
void OS_ClearReadySetCanContinue (void);

    
//------------------------------------------------------------------------------

#define _OS_RETURN_SAVE()                   OS_SchedRetPoint()
#define _OS_CLEAR_READY()                   OS_ClearReady()
#define _OS_CLEAR_READY_SET_CANCONTINUE()   OS_ClearReadySetCanContinue()
#define _OS_RETURN_NO_SAVE()                asm("   return                  ")
                                            //OS_SchedRetPointNoSave()

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Set FSR to point to current task
//------------------------------------------------------------------------------

/**
#if defined(_ROMSIZE) && (_ROMSIZE <= 0x800)

    extern void  _OS_SET_FSR_CUR_TASK_W (char);
    extern void  _OS_SET_FSR_CUR_TASK (void);

#else

    #define _OS_SET_FSR_CUR_TASK()      {_fsr =  (_FSR_TYPE)_OS_CurTask     ; _SET_TASK_IRP();}
    #define _OS_SET_FSR_CUR_TASK_W(w)   {_fsr = ((_FSR_TYPE)_OS_CurTask) + w; _SET_TASK_IRP();}

#endif
/**/

#define _OS_SET_FSR_CUR_TASK()      _fsr = (_FSR_TYPE)_OS_CurTask
#define _OS_SET_FSR_CUR_TASK_W(w)   {_fsr = (_FSR_TYPE)_OS_CurTask; asm(" dw 0x3100 +"#w);}






/************************************************************************************************
 *                                                                                              *
 *     Return from task to OS_Sched macros                                                      *
 *                                                                                              *
 ************************************************************************************************/


/*
 ********************************************************************************
 *                                                                              *
 *   Switch control routines:                                                   *
 *                                                                              *
 *    _OS_RETURN_SAVE()                   - exit from task and save context     *
 *                                                                              *
 *    _OS_RETURN_NO_SAVE()                - exit withowt saving return address  *
 *                                                                              *
 *    _OS_CLEAR_READY()                   - exit with saving address and        *
 *                                          clearing bReady                     *
 *                                                                              *
 *    _OS_CLEAR_READY_SET_CANCONTINUE()   - exit without saving address and     *
 *                                          clearing bReady and bCanContinue    *
 *                                                                              *
 ********************************************************************************
 */

















/************************************************************************************************
 *                                                                                              *
 *     Create, replace, reserve task specific services                                          *
 *                                                                                              *
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

#define OS_Task_Replace(priority, TaskName)                                     \
    {                                                                           \
        _OS_SET_FSR_CUR_TASK();                                                 \
        _indf = priority | 0x48; /* bEnable  bReady */                         \
        FSR1 = (__osa_uint16)TaskName;                                          \
        asm("   dw      0x0806          ");      /* movf    FSR1L, w    */      \
        asm("   dw      0x0018          ");      /* movwi   ++FSR0      */      \
        asm("   dw      0x0807          ");      /* movf    FSR1H, w    */      \
        asm("   dw      0x0018          ");      /* movwi   ++FSR0      */      \
        _OS_Return();                                                           \
    }


#define OS_Task_Replace_P(p,t)   OS_Task_Replace(p,t)   /*100300.3*/



/************************************************************************/
/*                                                                      */
/* Create task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Create(priority, TaskName)                          \
    {                                                               \
        _OS_Task_Create(priority, (OST_CODE_POINTER)TaskName);      \
    }

#define OS_Task_Create_M(priority, TaskName)                        \
    {                                                               \
        OS_Task_Reserve(TaskName);                                  \
        _OS_Task_Create(priority, (OST_CODE_POINTER)TaskName);      \
    }



/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_Reserve(TaskName)                                       \
        if (_pclath & 0x80) TaskName();                                 \







/************************************************************************************************
 *                                                                                              *
 *                                     I N T E R R U P T S                                      *
 *                                                                                              *
 ************************************************************************************************/

#ifndef     di
    #define di()    (GIE = 0)       // interrupt enable bit
#endif


extern  char OS_DI (void);
#define OS_RI(cSavedInt)    OSM_BEGIN {    if (cSavedInt & 0x80) _gie = 1; } OSM_END
#define OS_EI()             OSM_BEGIN {    _gie = 1; } OSM_END


#ifdef OS_ENABLE_INT

extern void OS_DI_INT (void);
extern void OS_RI_INT (void);

#endif

/************************************************************************************************
 *                                                                                              *
 *                                     C O N D I T I O N S                                      *
 *                                                                                              *
 ************************************************************************************************/

extern void _OS_CheckEvent (__osa_word);

#define _OS_CHECK_EVENT(event) _OS_CheckEvent(event)






#endif







